// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2005 - 2006 Simon Goodall

/*
 * TODO: Save the overrides (or all data?) to a resources file in current dir.
 * Or at least same dir as .jar file is exe from.
 */

package WFUT;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

/**
 * The Options class stores the user preferences for WFUT. It also sets the
 * default parameters. This is accessible only through the getInstance() method.
 * 
 */
public class Options {
  // Config item keys
  public static final String KEY_serverRoot = "serverRoot";
  public static final String KEY_prefix = "prefix";
  public static final String KEY_proxyHost = "proxyHost";
  public static final String KEY_proxyPort = "proxyPort";
  public static final String KEY_useProxy = "useProxy";
  public static final String KEY_bufferSize = "bufferSize";

  // Default option values
  private static final String DEFAULT_serverRoot = "http://white.worldforge.org/downloads/WFUT/";
  private static final String DEFAULT_prefix = "./";
  private static final String DEFAULT_proxyHost = "";
  private static final String DEFAULT_proxyPort = "8080";
  private static final String DEFAULT_useProxy = "false";
  private static final String DEFAULT_bufferSize = "8192";
  
  private static final String WFUT_CONFIG = "wfut_config";
  
  private static Options instance = new Options();
  private PropertyResourceBundle m_props = null;
  private Map m_overrides = new HashMap();
  

  /**
   * Returns the static instance of the Options class.
   * 
   * @return Returns the gobal instance.
   */
  public static Options getInstance() {
    return instance;
  }

  /** Creates a new instance of Options 
   * @throws IOException 
   * @throws FileNotFoundException */
  private Options() {
    try {
      // First see is a file called wfut_config.properties exists in the currrent dir
      File f = new File(WFUT_CONFIG + ".properties");
      if (f.exists()) {
        m_props = new PropertyResourceBundle(new FileInputStream(f));
      } else {
        // Else use the normal classloader stuff to find it
        m_props = (PropertyResourceBundle) ResourceBundle.getBundle(WFUT_CONFIG);
      }
    } catch (MissingResourceException mre) {
      System.err.println(mre);
    } catch (FileNotFoundException e) {
      System.err.println(e);
    } catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * Wrapper to get on m_prefs
   * 
   * @param key
   *          The name of the property
   * @param defVal
   *          The default value for this parameter
   * @return Return the stored value, or the default value if the property does
   *         not exist.
   */
  protected String get(final String key, final String defVal) {
    // Check overrides first
    Object o = m_overrides.get(key);
    if (o != null && o instanceof String) {
      return (String) o;
    }
    
    // Check properties for value
    final String pval = System.getProperty(key);
    if (pval != null) {
      m_overrides.put(key, pval);
      return pval;
    }
        
    // Check properties file
    if (m_props == null)
      return defVal;
    try {
      return m_props.getString(key);
    } catch (MissingResourceException mre) {
      return defVal;
    }
  }

  /**
   * Wrapper to set on m_prefs
   * 
   * @param key
   *          The name of the property.
   * @param value
   *          The value of the property
   */
  protected void set(final String key, final String value) {
    m_overrides.put(key, value);
  }

  /**
   * Return the current server root.
   * 
   * @return The root URL to the server.
   */
  public String getServerRoot() {
    return get(KEY_serverRoot, DEFAULT_serverRoot);
  }

  public void setServerRoot(final String str) {
    set(KEY_serverRoot, str);
  }

  public String getPrefix() {
    return get(KEY_prefix, DEFAULT_prefix);
  }

  public void setPrefix(final String str) {
    set(KEY_prefix, str);
  }

  public String getProxyHost() {
    return get(KEY_proxyHost, DEFAULT_proxyHost);
  }

  public void setProxyHost(final String str) {
    set(KEY_proxyHost, str);
  }

  public String getProxyPort() {
    return get(KEY_proxyPort, DEFAULT_proxyPort);
  }

  public void setProxyPort(final String str) {
    set(KEY_proxyPort, str);
  }

  public String getUseProxy() {
    return get(KEY_useProxy, DEFAULT_useProxy);
  }

  public void setUseProxy(final String str) {
    set(KEY_useProxy, str);
  }

  public String getBufferSize() {
    return get(KEY_bufferSize, DEFAULT_bufferSize);
  }

  public void setBufferSize(final String str) {
    set(KEY_bufferSize, str);
  }

  // Some explicit casts for non-string data

  public boolean getUseProxyBoolean() {
    return (get(KEY_useProxy, DEFAULT_useProxy).equals("true")) ? true : false;
  }

  public void setUseProxy(boolean b) {
    set(KEY_useProxy, (b) ? "true" : "false");
  }

  public int getProxyPortInteger() {
    return Integer.parseInt(get(KEY_proxyPort, DEFAULT_proxyPort));
  }

  public void setProxyPort(int i) {
    set(KEY_proxyPort, Integer.toString(i));
  }

  public int getBufferSizeInteger() {
    return Integer.parseInt(get(KEY_bufferSize, DEFAULT_bufferSize));
  }

  public void setBufferSize(int i) {
    set(KEY_bufferSize, Integer.toString(i));
  }
}
